<?php
namespace Cashbene\GatewayMagento\Controller;

use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\Core\Exception\NotFoundHttpException;
use Cashbene\Core\Exception\ParamMismatchException;
use Cashbene\Core\Exception\ParamMissingException;
use Cashbene\Core\Service\HttpExceptionService;
use Cashbene\Core\Service\Response;
use Cashbene\Core\Service\Validator;
use Cashbene\GatewayMagento\Service\GatewayService;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\Core\Service\SerializerService;
use Magento\Framework\Controller\ResultInterface;

abstract class CashbeneAbstractRestController
{
    public static $cashbeneGateway;

    /** @var \Symfony\Component\Serializer\Serializer */
    public static $serializer;

    /** @var LoggerInterface */
    protected $logger;

    public function __construct(
        LoggerInterface $logger,
        GatewayService $gatewayService,
        Context $context
    )
    {
        $this->logger = $logger;
        $this->gatewayService = $gatewayService->getGateway();
        $this->context = $context;
    }

    public function execute() {
        self::$cashbeneGateway = $this->gatewayService;
        self::$serializer = SerializerService::getSerializer();

        $request = Request::createFromGlobals();
        $action = $request->get('action');

        $httpMethod = strtoupper($request->getMethod());
        $actions = $httpMethod . '_ACTIONS';
        $reflection = new \ReflectionClass(get_class($this));
        $actions = $reflection->getConstant($actions);

        if (!$action || !in_array($action, $actions)) {

            $locale = self::$cashbeneGateway->configuration
                ->getLanguage();

            $validator = new Validator(
                new HttpExceptionService(),
                $locale
            );

            if (!$action) {
                return $this->error(
                    $validator->translateException(
                        new ParamMissingException("Param: 'action' missing.")
                    )
                );
            } else if (!in_array($action, $actions)) {
                return $this->error(
                    $validator->translateException(
                        new ParamMismatchException("Param: 'action' mismatch.")
                    )
                );
            }
        }

        switch ($httpMethod) {
            case 'GET':
                $response = $this->processGetRequest($action, $request);
                break;
            case 'POST':
                $response = $this->processPostRequest($action, $request);
                break;
            case 'PATCH':
            case 'PUT':
                $response = $this->processPutRequest($action, $request);
                break;
            case 'DELETE':
                $response = $this->processDeleteRequest($action, $request);
                break;
        }

        if (empty($response)) {
            $this->error(new NotFoundHttpException("Not found."));
        }
        return $response;
    }

    abstract protected function processGetRequest(string $action, Request $request);
    abstract protected function processPostRequest(string $action, Request $request);
    abstract protected function processPutRequest(string $action, Request $request);
    abstract protected function processDeleteRequest(string $action, Request $request);

    /**
     * Returns JsonResponse object based on the contents of a given exception.
     *
     * @param HttpExceptionInterface $httpException
     * @param $code
     * @return JsonResponse
     */
    public function error(HttpExceptionInterface $httpException, $code = null)
    {
        $responseBody = Response::prepareSimpleErrorResponseBody($httpException);
        $response = $this->context->getResultFactory()->create(ResultFactory::TYPE_JSON);

        if (is_null($code)) {
            $code = $httpException->getHttpCode();
        }

        return $response->setHttpResponseCode($code)->setData($responseBody);
    }

    /**
     * Returns JsonResponse object with given data and http status code.
     *
     * @param array|object|null $data
     * @param $code
     * @return JsonResponse
     */
    public function success($data = null, $code = 200)
    {
        $responseBody = Response::prepareSimpleSuccessResponseBody($data);

        $response = $this->context->getResultFactory()->create(ResultFactory::TYPE_JSON);
        return $response->setHttpResponseCode($code)->setData($responseBody);
    }
}
