<?php
namespace Cashbene\GatewayMagento\Controller\Index;

use Magento\Framework\App\ActionInterface;
use Cashbene\Core\Exception\HttpException;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\Core\Utils\CredentialsSession;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\GatewayMagento\Controller\CashbeneAbstractRestController;
use Cashbene\GatewayMagento\Service\CheckoutService;
use Cashbene\GatewayMagento\Utils\Shop as ShopUtils;
use Cashbene\GatewayMagento\Service\GatewayService;
use Magento\Framework\App\Action\Context;
use Psr\Log\LoggerInterface;

class Checkout extends CashbeneAbstractRestController implements ActionInterface
{
    const POST_ACTIONS = ['calculate', 'executeCheckout', 'calculateWholeCart', 'executeCheckoutCart'];
    const GET_ACTIONS = ['checkoutData'];

    public function __construct(
        ShopUtils $shopUtils,
        GatewayService $gatewayService,
        Context $context,
        CheckoutService $checkoutService,
        LoggerInterface $logger
    )
    {
        $this->shopUtils = $shopUtils;
        $this->checkoutService = $checkoutService;
        parent::__construct($logger, $gatewayService, $context);
    }

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'checkoutData':
                return $this->getCheckoutData($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::getCheckoutData()
     */
    private function getCheckoutData(Request $request) {
        try {
            $data = self::$cashbeneGateway->checkoutContext()->getCheckoutData(CredentialsSession::getCredentials(self::$cashbeneGateway), $request->get('checkoutId'));
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }


    protected function processPostRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'calculate':
                return $this->calculateCart($request);
            case 'executeCheckout':
                return $this->checkoutCart($request);
            case 'calculateWholeCart':
                return $this->calculateWholeCart($request);
            case 'executeCheckoutCart':
                return $this->checkoutWholeCart($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::calculateCartValue()
     */
    private function calculateCart(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);

            $product = $this->shopUtils->getProduct(
                $this->shopUtils->getMagentoProduct($params['productId']),
                isset($params['variationId']) ? $this->shopUtils->getMagentoProduct($params['variationId']) : null
            );

            $product->quantity = $params['quantity'] ?? 1;
            $product->totalPrice = clone $product->unitPrice;
            $product->displayPrice = clone $product->unitPrice;

            $product->totalPrice->amount = $product->unitPrice->amount * $product->quantity;
            $product->displayPrice->amount = $product->unitPrice->amount * $product->quantity;

            $cashbeneUser = self::$cashbeneGateway->userContext()->getUserData(
                CredentialsSession::getCredentials(self::$cashbeneGateway)
            );

            $calcOrder = $this->checkoutService->prepareOrder(
                [$product],
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId(),
                null,
                $cashbeneUser
            );

            $data = self::$cashbeneGateway->checkoutContext()->calculateCartValue(
                CredentialsSession::getCredentials(self::$cashbeneGateway),
                $calcOrder
            );
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::calculateCartValue()
     */
    private function calculateWholeCart(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);
            $products = $this->checkoutService->getCartProducts();

            if (empty($products)) {
                return $this->error(new HttpException(404, "Cart products not found."));
            }

            $cashbeneUser = self::$cashbeneGateway->userContext()->getUserData(
                CredentialsSession::getCredentials(self::$cashbeneGateway)
            );

            $calcOrder = $this->checkoutService->prepareOrder(
                $products,
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId(),
                null,
                $cashbeneUser
            );

            // not implemented
//            $calcOrder->discount = $this->checkoutService->getDiscount();

            $data = self::$cashbeneGateway->checkoutContext()->calculateCartValue(
                CredentialsSession::getCredentials(self::$cashbeneGateway),
                $calcOrder
            );
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::checkout()
     */
    private function checkoutCart(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);
            $product = $this->shopUtils->getMagentoProduct($params['productId']);
            $productVariation = isset($params['variationId']) ? $this->shopUtils->getMagentoProduct($params['variationId']) : null;

            // Prepare product
            $cashbeneProduct = $this->shopUtils->getProduct($product, $productVariation);
            $cashbeneProduct->quantity = $params['quantity'] ?? 1;

            $cashbeneProduct->totalPrice = clone $cashbeneProduct->unitPrice;
            $cashbeneProduct->totalPrice->amount = $cashbeneProduct->unitPrice->amount * $cashbeneProduct->quantity;

            $cashbeneUser = self::$cashbeneGateway->userContext()->getUserData(
                CredentialsSession::getCredentials(self::$cashbeneGateway)
            );

            // Create Magento Order
            $magentoOrder = $this->checkoutService->createMagentoOrder(
                $cashbeneUser,
                [$cashbeneProduct],
                $params['shippingAddressId'],
                $params['shippingMethodId']
            );

            $externalOrderId = $magentoOrder->getId() . '-M-' .
            (
            defined('CONTAINER_TAG')
                ? CONTAINER_TAG . '-' . self::gen_uuid()
                : self::gen_uuid()
            );

            // Prepare order object
            $checkoutOrder = $this->checkoutService->prepareOrder(
                [$cashbeneProduct],
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId(),
                $externalOrderId,
                $cashbeneUser
            );

            $checkoutId = self::$cashbeneGateway->checkoutContext()->checkout(CredentialsSession::getCredentials(self::$cashbeneGateway), $checkoutOrder);
            $data = self::$cashbeneGateway->checkoutContext()->getCheckoutData(CredentialsSession::getCredentials(self::$cashbeneGateway), $checkoutId);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        } catch (\Exception $exception) {
            return $this->error(new HttpException($exception->getCode() ?: 500, $exception->getMessage()));
        }

        return $this->success($data, 201);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::checkout()
     */
    private function checkoutWholeCart(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);
            $products = $this->checkoutService->getCartProducts();
            if (empty($products)) {
                return $this->error(new HttpException(404, "Cart products not found."));
            }

            $cashbeneUser = self::$cashbeneGateway->userContext()->getUserData(
                CredentialsSession::getCredentials(self::$cashbeneGateway)
            );

            // Create Magento Order
            $magentoOrder = $this->checkoutService->createMagentoOrder(
                $cashbeneUser,
                $products,
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                true
            );

            $externalOrderId = $magentoOrder->getId() . '-M-' .
                (
                defined('CONTAINER_TAG')
                    ? CONTAINER_TAG . '-' . self::gen_uuid()
                    : self::gen_uuid()
                );

            // Prepare order object
            $checkoutOrder = $this->checkoutService->prepareOrder(
                $products,
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId(),
                $externalOrderId,
                $cashbeneUser
            );

            $checkoutId = self::$cashbeneGateway->checkoutContext()->checkout(CredentialsSession::getCredentials(self::$cashbeneGateway), $checkoutOrder);
            $data = self::$cashbeneGateway->checkoutContext()->getCheckoutData(CredentialsSession::getCredentials(self::$cashbeneGateway), $checkoutId);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        } catch (\Exception $exception) {
            return $this->error(new HttpException($exception->getCode() ?: 500, $exception->getMessage()));
        }

        return $this->success($data, 201);
    }

    protected function processPutRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    protected function processDeleteRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    private function gen_uuid() {
        return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
            // 32 bits for "time_low"
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),

            // 16 bits for "time_mid"
            mt_rand( 0, 0xffff ),

            // 16 bits for "time_hi_and_version",
            // four most significant bits holds version number 4
            mt_rand( 0, 0x0fff ) | 0x4000,

            // 16 bits, 8 bits for "clk_seq_hi_res",
            // 8 bits for "clk_seq_low",
            // two most significant bits holds zero and one for variant DCE1.1
            mt_rand( 0, 0x3fff ) | 0x8000,

            // 48 bits for "node"
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff )
        );
    }
}
