<?php
namespace Cashbene\GatewayMagento\Controller\Index;

use Cashbene\Core\Utils\CredentialsSession;
use Cashbene\GatewayMagento\Service\CheckoutService;
use Magento\Framework\App\ActionInterface;
use Cashbene\Core\Exception\HttpException;
use Cashbene\Core\Exception\InvalidOrderStatusChangeException;
use Cashbene\Core\Exception\InvalidPaidOrderStatusChangeException;
use Cashbene\Core\Exception\InvalidRequestSignatureException;
use Cashbene\Core\Exception\SignatureHeaderMissingException;
use Cashbene\Core\Exception\ValidationFailedErrorException;
use Cashbene\GatewayMagento\Service\GatewayService;
use Cashbene\GatewayMagento\Service\OrderStatusService;
use Cashbene\GatewayMagento\Utils\Shop as ShopUtils;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\PaymentMagento\Model\Payment\PaymentMethod;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\GatewayMagento\Controller\CashbeneAbstractRestController;
use Magento\Framework\App\Action\Context;
use Psr\Log\LoggerInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;

class Shop extends CashbeneAbstractRestController implements ActionInterface, CsrfAwareActionInterface
{
    public function __construct(
        ShopUtils $shopUtils,
        GatewayService $gatewayService,
        Context $context,
        LoggerInterface $logger,
        OrderStatusService $orderStatusService,
        CheckoutService $checkoutService
    )
    {
        $this->shopUtils = $shopUtils;
        $this->checkoutService = $checkoutService;
        $this->orderStatusService = $orderStatusService;
        parent::__construct($logger, $gatewayService, $context);
    }

    public function createCsrfValidationException(RequestInterface $request): ? InvalidRequestException
    {
        return null;
    }

    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }

    const POST_ACTIONS = ['webhook', 'simulateWebhook'];
    const GET_ACTIONS = ['merchantShippingMethods', 'product', 'regions', 'cartProducts', 'verificationCode'];
    const CYPRESS_KEY = 'Cypress_Cashbene_Simulation_Secret_Key';

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'merchantShippingMethods':
                return $this->getMerchantShippingMethods();
            case 'product':
                return $this->getProduct($request);
            case 'regions':
                return $this->getRegions($request);
            case 'cartProducts':
                return $this->getCartProducts($request);
            case 'verificationCode':
                return $this->getVerificationCode($request);
        }
    }

    /**
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see ShopContext::getMerchantShippingMethods()
     */
    private function getMerchantShippingMethods() {
        try {
            $shippingMethods = self::$cashbeneGateway->shopContext()->getMerchantShippingMethods();
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($shippingMethods);
    }

    /**
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see ShopContext::getVerificationCode()
     */
    private function getVerificationCode($request) {
        if (hash('sha256', self::CYPRESS_KEY) != $request->headers->get('Cypress-Key')) {
            return $this->error(
                new HttpException(404, "Not Found.")
            );
        }

        try {
            $phoneNumber = $request->query->get('phoneNumber');
            $code = self::$cashbeneGateway->shopContext()->getVerificationCode($phoneNumber);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($code);
    }

    /**
     * Return product details by product id
     *
     * @param Request $request
     * @return JsonResponse
     * @throws \Exception|\Symfony\Component\Serializer\Exception\ExceptionInterface
     * @see ShopUtils::getProduct()
     */
    private function getProduct(Request $request) {
        try {
            $productId = (int) $request->query->get('product_id');
            $variationId = $request->query->get('super_attribute', 0);
            $qty = $request->query->get('qty') ?? 1;

            $product = $this->shopUtils->getProduct(
                $this->shopUtils->getMagentoProduct($productId),
                $this->shopUtils->getMagentoProductConfigurableVariation($productId, $variationId)
            );

            $product->quantity = $qty;
            $product->totalPrice = clone  $product->unitPrice;
            $product->totalPrice->amount = $product->unitPrice->amount * $product->quantity;

        } catch (\Exception $e) {
            return $this->error(new HttpException(500, "Unknown error."));
        }

        return $this->success($product);
    }

    /**
     * Return product details by product id
     *
     * @param Request $request
     * @return JsonResponse
     * @throws \Exception|\Symfony\Component\Serializer\Exception\ExceptionInterface
     * @see ShopUtils::getProduct()
     */
    private function getCartProducts(Request $request) {
        try {
            $products = $this->checkoutService->getCartProducts();
        } catch (\Exception $e) {
            return $this->error(new HttpException(500, "Unknown error."));
        }

        return $this->success($products);
    }

    private function getRegions(Request $request) {
        $countryCode = $request->query->get('country', 'PL');
        $regions = $this->shopUtils->getRegionsByCountryCode($countryCode);

        return $this->success($regions);
    }

    protected function processPostRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'webhook':
                return $this->webhook($request);
            case 'simulateWebhook':
                return $this->simulateWebhook($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    private function webhook(Request $request)
    {
        try {
            $this->logWebhookRequest($request);
            $jsonParams = json_decode($request->getContent(), true);

            if (!$request->headers->get('x_signature')) {
                $this->logWebhookError("Header: 'X-Signature' doesn't exists.");

                return $this->error(
                    new SignatureHeaderMissingException("Missing signature header.")
                );
            }

            $signatureChecker = self::$cashbeneGateway->getSignatureChecker();
            $correctlySignature = $signatureChecker->createSignature($jsonParams);
            $isValidSignatureRequest = $signatureChecker->compareSignatures(
                $request->headers->get('x_signature'),
                $correctlySignature
            );

            if (!$isValidSignatureRequest) {
                $this->logWebhookError("The request signature is invalid.");

                return $this->error(
                    new InvalidRequestSignatureException("The request signature is invalid.")
                );
            }

            // Checks whether the required parameters exist
            $validationsError = [];

            if (!isset($jsonParams['status'])) {
                $this->logWebhookError("Param: 'status' doesn't exists.");

                $validationsError[] = [
                    "field" => "status",
                    "message" => "must not be null"
                ];
            }

            if (!isset($jsonParams['extOrderId'])) {
                $this->logWebhookError("Param: 'extOrderId' doesn't exists.");

                $validationsError[] = [
                    "field" => "extOrderId",
                    "message" => "must not be null"
                ];
            }

            if (!empty($validationsError)) {
                return $this->error(
                    (new ValidationFailedErrorException("Request parameters constraints violation"))
                        ->setErrors($validationsError)
                );
            }

            // Checks whether the status and order are valid
            $statusId = $this->orderStatusService->getMagentoStatusByCashbeneStatus($jsonParams['status']);

            if ($statusId === false) {
                $this->logWebhookError("Unknown status: '{$jsonParams['status']}'");

                $validationsError[] = [
                    "field" => "status",
                    "message" => "unknown status: '{$jsonParams['status']}', must be 'SUCCESSFUL' or 'FAILED'"
                ];
            }

            $orderId = (int) substr(
                $jsonParams['extOrderId'],
                0,
                strpos($jsonParams['extOrderId'], '-')
            );

            $order = $this->orderStatusService->getOrder($orderId);
            if ($order === false || $order->getPayment()->getMethod() !== PaymentMethod::PAYMENT_NAME) {
                $this->logWebhookError("Unknown order by ID: '{$jsonParams['extOrderId']}'");

                $validationsError[] = [
                    "field" => "extOrderId",
                    "message" => "order with the number '$orderId' does not exist"
                ];
            }

            if (!empty($validationsError)) {
                return $this->error(
                    (new ValidationFailedErrorException("Request parameters constraints violation"))
                        ->setErrors($validationsError)
                );
            }

            $orderStateId = $order->getStatus();
            $orderStates = $this->orderStatusService->getAllOrderStatusKeys();

            if (
                $orderStateId == $orderStates['pending']
                || $orderStateId == $orderStates['processing']
            ) {
                $this->orderStatusService->orderSetStatus($orderId, $statusId);
            } elseif (
                $orderStateId == $orderStates['complete']
                || $orderStateId == $orderStates['closed']
            ) {
                return $this->error(new InvalidPaidOrderStatusChangeException("Invalid status change for a paid order."));
            } else {
                return $this->error(new InvalidOrderStatusChangeException("Invalid status change for an order."));
            }

            return $this->success();

        } catch (\Exception $e) {
            $this->logWebhookError(
                print_r([
                    'message' => $e->getMessage(),
                    'file' => $e->getFile(),
                    'line' => $e->getLine(),
                    'code' => $e->getCode(),
                    'trace' => $e->getTrace()
                ], true)
            );

            return $this->error(new HttpException(500, "Unknown error."));
        }
    }

    private function simulateWebhook($request) {
        if (hash('sha256', self::CYPRESS_KEY) != $request->get_header('Cypress-Key')) {
            return $this->error(
                new HttpException(404, "Not Found.")
            );
        }

        $availableStatus = ['COMPLETED', 'CANCELED'];
        try {
            $jsonParams = json_decode($request->getContent(), true);
            $order = [];

            if (!isset($jsonParams['status']) || !in_array($jsonParams['status'], $availableStatus)) {
                return $this->error(
                    new HttpException(400, "Status missing or not match requirements.")
                );
            }

            if (!isset($jsonParams['orderId'])) {
                return $this->error(
                    new HttpException(400, "Missing parameter extOrderId.")
                );
            }

            $order['order']['extOrderId'] = 'CK-'.$jsonParams['orderId'];
            $order['order']['status'] = $jsonParams['status'];

            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);
            $code = self::$cashbeneGateway->shopContext()->simulateWebhook($credentials, $order);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($code);
    }

    /**
     * @param string $message
     * @return void
     */
    private function logWebhookError(string $message)
    {
        $this->logger->error($message);
    }

    /**
     * @param Request $request
     * @return void
     */
    private function logWebhookRequest(Request $request)
    {
        $this->logger->info(
            print_r([
                'json_params' => json_decode($request->getContent(), true),
                'headers' => $request->headers
            ], true)
        );
    }

    protected function processPutRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    protected function processDeleteRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }
}
