<?php
namespace Cashbene\GatewayMagento\Controller\Index;

use Magento\Framework\App\ActionInterface;
use Cashbene\Core\Dto\ShippingMethod;
use Cashbene\Core\Dto\ShippingMethodOutput;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\Core\Utils\CredentialsSession;
use Cashbene\GatewayMagento\Controller\CashbeneAbstractRestController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class User extends CashbeneAbstractRestController implements ActionInterface
{
    const POST_ACTIONS = ['signIn', 'shipmentAddresses'];
    const PATCH_ACTIONS = ['shipmentAddresses'];
    const GET_ACTIONS = ['users', 'shipmentAddresses'];
    const DELETE_ACTIONS = ['shipmentAddresses'];

    protected function processPostRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'signIn':
                return $this->signIn($request);
            case 'shipmentAddresses':
                return $this->addShipmentAddress($request);
        }
    }

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'users':
                return $this->getUserData();
            case 'shipmentAddresses':
                return $this->getShipmentAddresses($request);
        }
    }

    protected function processDeleteRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'shipmentAddresses':
                return $this->deleteShipmentAddress($request);
        }
    }

    /**
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::getUserData()
     */
    private function getUserData() {
        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);
            $data = self::$cashbeneGateway
                ->userContext()
                ->getUserData($credentials);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::getShipmentAddresses()
     */
    private function getShipmentAddresses(Request $request) {
        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);

            /** @var \Cashbene\Core\Dto\User $userData */
            $userData = self::$cashbeneGateway
                ->userContext()
                ->getUserData($credentials);

            $data = self::$cashbeneGateway
                ->userContext()
                ->getShipmentAddresses($credentials, $userData->uuid, $request->get('type') ?? null);

            // check if data was updated if yes get again
            if ($this->updateAddressesState($data, $credentials, $userData) > 0) {
                $data = self::$cashbeneGateway
                    ->userContext()
                    ->getShipmentAddresses($credentials, $userData->uuid, $request->get('type') ?? null);
            }

        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    private function updateAddressesState(array $shippings, $credentials, $userData) {
        /** @var ShippingMethodOutput $shipping */
        $fixed = 0;
        foreach ($shippings as $shipping) {
            if ($shipping->type === ShippingMethodOutput::TYPE_POINT && $shipping->address->state === null) {
                $fixed++;
                $content = file_get_contents("https://api-pl-points.easypack24.net/v1/points?status=Operating&name=".$shipping->pointId);
                $content = json_decode($content);
                if ($content->count > 0) {
                    $data = clone $shipping;
                    $data->shipmentAddressId = null;

                    /** @var ShippingMethod $shippingMethod */
                    $shippingMethod = self::$serializer->denormalize(
                        $data, ShippingMethod::class, 'array'
                    );
                    $shippingMethod->address->state = $content->items[0]->address_details->province;

                    // update element - delete and add new
                    self::$cashbeneGateway
                        ->userContext()
                        ->deleteShipmentAddress($credentials, $userData->uuid, $shipping->shipmentAddressId);

                    self::$cashbeneGateway
                        ->userContext()
                        ->addShipmentAddress($credentials, $shippingMethod);
                } else {
                    // delete element if not found
                    self::$cashbeneGateway
                        ->userContext()
                        ->deleteShipmentAddress($credentials, $userData->uuid, $shipping->shipmentAddressId);
                }
            }
        }

        return $fixed;
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::signIn()
     */
    private function signIn(Request $request) {
        $data = json_decode($request->getContent(), true);

        try {
            $clientCredentials  = self::$cashbeneGateway->userContext()->signIn($data['email'], $data['password']);
            CredentialsSession::saveCredentials($clientCredentials);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success();
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::addShipmentAddress()
     */
    private function addShipmentAddress(Request $request) {
        $data = json_decode($request->getContent(), true);

        /** @var ShippingMethod $shippingMethod */
        $shippingMethod = self::$serializer->denormalize(
            $data, ShippingMethod::class, 'array'
        );

        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);
            self::$cashbeneGateway
                ->userContext()
                ->addShipmentAddress($credentials, $shippingMethod);

        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success(null, 201);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::deleteShipmentAddress()
     */
    public function deleteShipmentAddress(Request $request)
    {
        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);

            /** @var \Cashbene\Core\Dto\User $userData */
            $userData = self::$cashbeneGateway
                ->userContext()
                ->getUserData($credentials);

            self::$cashbeneGateway
                ->userContext()
                ->deleteShipmentAddress($credentials, $userData->uuid, $request->get('addressUuid'));
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success(null, 204);
    }

    protected function processPutRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'shipmentAddresses':
                return $this->updateShipmentAddress($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::updateShipmentAddress()
     */
    public function updateShipmentAddress(Request $request)
    {
        $data = json_decode($request->getContent(), true);

        /** @var ShippingMethod $shippingMethod */
        $shippingMethod = self::$serializer->denormalize(
            $data, ShippingMethod::class, 'array'
        );

        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);

            /** @var \Cashbene\Core\Dto\User $userData */
            $userData = self::$cashbeneGateway
                ->userContext()
                ->getUserData($credentials);

            self::$cashbeneGateway
                ->userContext()
                ->updateShipmentAddress($credentials, $userData->uuid, $request->get('addressUuid'), $shippingMethod);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success();
    }
}
