<?php

namespace Cashbene\GatewayMagento\Service;

use Cashbene\Core\Dto\Shop\Price;
use Cashbene\GatewayMagento\Utils\Shop as ShopUtils;
use Magento\Sales\Api\OrderRepositoryInterface;
use Cashbene\Core\Dto\Checkout\CheckoutData;
use Cashbene\Core\Dto\Checkout\CheckoutOrder;
use Cashbene\Core\Dto\ShippingMethodOutput;
use Cashbene\Core\Dto\Shop\Product;
use Cashbene\Core\Dto\User;
use Cashbene\Core\Utils\CredentialsSession;
use Cashbene\GatewayMagento\Controller\CashbeneAbstractRestController;
use Cashbene\GatewayMagento\Helper\Data;
use Magento\Directory\Model\ResourceModel\Region\CollectionFactory;
use Cashbene\GatewayMagento\Utils\Lang;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Checkout\Model\Session;
use Magento\Framework\Pricing\PriceCurrencyInterface;

class CheckoutService {

    public function __construct(
        Data $dataHelper,
        CollectionFactory $collectionFactory,
        ConfigurationService $configurationService,
        GatewayService $gatewayService,
        OrderRepositoryInterface $orderRepository,
        InpostService $inpostService,
        StoreManagerInterface $storeManager,
        Session $session,
        ShopUtils $shopUtils,
        PriceCurrencyInterface $priceCurrency
    ) {
        $this->currency = $storeManager->getStore()->getCurrentCurrency()->getCurrencyCode();
        $this->dataHelper = $dataHelper;
        $this->regionCollectionFactory = $collectionFactory;
        $this->configurationService = $configurationService;
        $this->gatewayService = $gatewayService;
        $this->orderRepository = $orderRepository;
        $this->inpostService = $inpostService;
        $this->session = $session;
        $this->shopUtils = $shopUtils;
        $this->priceCurrency = $priceCurrency;
    }

	/**
	 * @param Product[] $products
	 * @param string $shippingAddressId
	 * @param string $shippingMethodId
	 * @param string $merchantId
	 * @param string|null $externalOrderId
	 *
	 * @return CheckoutOrder
	 */
    public function prepareOrder(
		array $products,
		string $shippingAddressId,
		string $shippingMethodId,
		string $merchantId,
		?string $externalOrderId = null,
        $user = null
	) {
		$order = new CheckoutOrder();
        $order->shippingAddressId = $shippingAddressId;

        $checkoutData = new CheckoutData();
        $checkoutData->shippingMethodId = $shippingMethodId;

        if ($externalOrderId !== null) {
            $checkoutData->externalOrderId = $externalOrderId;
        }

        $prepareAddress = self::prepareAddress($user, $shippingAddressId);
        $tempOrder = $this->prepareOrderData($user, $products, $shippingMethodId, $prepareAddress, 0);
        $checkoutData->products = $this->reCalculateProducts($products, $tempOrder);

        $order->checkoutData = (object)[
            $merchantId => $checkoutData
        ];

        return $order;
	}

	/**
	 * @param Product[] $products
	 * @return Product[]
	 */
	private function reCalculateProducts(array &$products, $tempOrder = [])
	{
        $quote = $this->dataHelper->prepareQuote($tempOrder);
        $itemsArray = [];
        foreach ($quote->getItems() as $item) {
            $itemsArray[$item->getSku()]['price'] = $item->getPrice();
            $itemsArray[$item->getSku()]['priceTax'] = $item->getPriceInclTax();
            $itemsArray[$item->getSku()]['taxRate'] = $this->dataHelper->calculateVatFromPrices($item->getPriceInclTax(), $item->getPrice());
        }

		foreach ($products as $product) {
            if (isset($itemsArray[$product->sku])) {
                $product->unitPrice->amount = $itemsArray[$product->sku]['priceTax'];
                $product->unitPrice->setTaxRate($itemsArray[$product->sku]['taxRate']);
                $product->unitPrice->setHasTax(true);
            }
		}

		return $products;
	}

    /**
     * @param string $region
     * @return string[]
     */
    public function getRegionCode(string $region): array
    {
        $regionCode = $this->regionCollectionFactory->create()
            ->addRegionNameFilter($region)
            ->getFirstItem()
            ->toArray();
        return $regionCode;
    }

	public function createMagentoOrder(
		User $user,
		array $products,
		string $shippingAddressId,
		string $shippingMethodId,
        bool $isCart = false
	) {
        $prepareAddress = self::prepareAddress($user, $shippingAddressId);
        $tempOrder = $this->prepareOrderData($user, $products, $shippingMethodId, $prepareAddress);
        if ($isCart) {
            $order = $this->dataHelper->createMageOrder($tempOrder, $this->getCart());
        } else {
            $order = $this->dataHelper->createMageOrder($tempOrder);
        }

        if (isset($prepareAddress['point_id'])) {
            $this->setOrderParcelLocker($order->getIncrementId(), $prepareAddress);
        }

		return $order;
	}

    private function prepareOrderData($user, $products, $shippingMethodId, $prepareAddress, $createUser = 1)
    {
        $billingAddress = isset($prepareAddress['point_id']) ? $this->getBillingAddress($user) : $prepareAddress;
        $shippingMethods = json_decode($this->configurationService->getConfigValue('shipping_method'), true);
        $cashbeneShippingMethod = $this->gatewayService->getGateway()->shopContext()->getMerchantShippingMethod($shippingMethodId);

        $tempOrder = [
            'currency_id' => $this->currency,
            'email' => $user->email, //buyer email id
            'shipping_address' => $prepareAddress,
            'billingAddress' => $billingAddress,
            'shipping_method' => $shippingMethods[$shippingMethodId],
            'shipping_price' => $cashbeneShippingMethod->price->amount,
            'shipping_title' => $cashbeneShippingMethod->name,
        ];

        if ($createUser !== 1) {
            unset($tempOrder['email']);
        }

        foreach ($products as $product) {
            $tempOrder['items'][] = [
                'product_id' => $product->productId,
                'product_variation' => $product->externalProductId,
                'qty' => $product->quantity
            ];
        }

        return $tempOrder;
    }

    /**
     * Add parcel locker to order in chosen shipment integration
     *
     * @param $orderId
     * @param $address
     */
    private function setOrderParcelLocker($orderId, $address)
    {
        $lockerId = $address['point_id'];
        $lockerAddress1 = $address['street'];
        $lockerAddress2 = $address['postcode'].' '.$address['city'];
        $val = $lockerId . ', ' . $lockerAddress1 . ', ' . $lockerAddress2;

        $this->inpostService->addData($orderId, $val);
    }

    /**
     * @param User $user
     * @return array|null
     * @throws \Exception
     */
    public function getBillingAddress(User $user)
    {
        $gateway = CashbeneAbstractRestController::$cashbeneGateway;
        $addresses = $gateway->userContext()->getShipmentAddresses(
            CredentialsSession::getCredentials($gateway),
            $user->uuid,
            ShippingMethodOutput::TYPE_COURIER
        );

        foreach ($addresses as $address) {
            if ($address->defaultShipment === true) {
                return self::prepareAddressArray($user, $address);
            }
        }

		// If no default address, return first address
        foreach ($addresses as $address) {
            if ($address->type === ShippingMethodOutput::TYPE_COURIER) {
                return self::prepareAddressArray($user, $address);
            }
        }

        return null;
    }

    /**
     * @param User $user
     * @param $shipmentAddressId
     * @return array
     * @throws \Exception
     */
	private function prepareAddress(User $user, $shipmentAddressId, $additionalFields = false)
	{
		$gateway = CashbeneAbstractRestController::$cashbeneGateway;
		$address = $gateway->userContext()->getShipmentAddress(
			CredentialsSession::getCredentials($gateway),
			$user->uuid,
			$shipmentAddressId
		);

        if ($address->type === ShippingMethodOutput::TYPE_POINT) {
            $additionalFields = true;
        }

		return self::prepareAddressArray($user, $address, $additionalFields);
	}

    /**
     * @param User $user
     * @param ShippingMethodOutput $address
     * @return array
     */
    private function prepareAddressArray(User $user, $address, $additionalFields = false)
    {
        $region = $this->getRegionCode($address->address->state ?? 'Łódzkie');
        $data = [
            'firstname' => $user->firstName, //address Details
            'lastname' => $user->lastName,
            'street' => $address->address->streetName.' '.$address->address->streetNumber. $address->address->flatNumber ?? ' '.$address->address->flatNumber,
            'city' => $address->address->city,
            'country_id' => (strlen($address->address->country) > 2)
                ? Lang::getAlpha2Code($address->address->country)
                : strtoupper($address->address->country),
            'region_id' => $region['region_id'],
            'postcode' => $address->address->postal,
            'telephone' => $user->mobile ? $user->mobile->prefix.$user->mobile->number : null,
            'name' => $address->name
        ];

		if ($additionalFields) {
			$data['point_id'] = $address->pointId ?? '';
		}

		return $data;
    }

    public function getCart() {
        return $this->session->getQuote();
    }

    public function getCartProducts() {
        $items = $this->getCart()->getAllVisibleItems();

        $products = [];
        foreach ($items as $item) {
            $productId = (int) $item->getProductId();
            $variationId = key($item->getQtyOptions());
            $price = $item->getRowTotalInclTax();
            $quantity = (int) $item->getQty();

            $product = $this->shopUtils->getMagentoProduct($productId);
            $productVariation = isset($variationId) ? $this->shopUtils->getMagentoProduct($variationId) : null;

            // Prepare product
            $cashbeneProduct = $this->shopUtils->getProduct($product, $productVariation);
            $cashbeneProduct->quantity = $quantity;

            $cashbeneProduct->totalPrice = clone $cashbeneProduct->unitPrice;
            $cashbeneProduct->displayPrice = clone $cashbeneProduct->unitPrice;
            $cashbeneProduct->totalPrice->amount = $this->priceCurrency->round($price);
            $cashbeneProduct->displayPrice->amount = $this->priceCurrency->round($price);

            $products[] = $cashbeneProduct;
        }

        return $products;
    }

    public function getProductsDiscout() {
        $items = $this->getCart()->getAllVisibleItems();

        $discountAmount = 0;
        foreach ($items as $item) {
            $discountAmount += $item->getDiscountAmount();
        }

        return $discountAmount;
    }

    public function getDiscount() {
        $price = new Price();
        $price->amount = $this->getProductsDiscout();
        $price->currency = $this->currency;

        return $price;
    }
}
