<?php

namespace Cashbene\GatewayMagento\Utils;

use Magento\Directory\Model\ResourceModel\Region\CollectionFactory;
use Cashbene\Core\Dto\Shop\Attribute;
use Cashbene\Core\Dto\Shop\Price;
use Cashbene\Core\Dto\Shop\Product as DtoProduct;
use Cashbene\Core\Service\SerializerService;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Catalog\Helper\Image;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Catalog\Helper\Data as TaxHelper;

final class Shop {
    /** @var ProductRepository */
    private $productRepository;

    public function __construct(
        Configurable $configurableProduct,
        ProductRepositoryInterface $productRepository,
        Image $imageHelper,
        StoreManagerInterface $storeManager,
        CollectionFactory $collectionFactory,
        TaxHelper $taxHelper
    )
    {
        $this->productRepository = $productRepository;
        $this->configurableProduct = $configurableProduct;
        $this->imageHelper = $imageHelper;
        $this->storeManager = $storeManager;
        $this->regionCollectionFactory = $collectionFactory;
        $this->taxHelper = $taxHelper;
    }

    /**
     * @param Product $product
     * @param $country
     * @return DtoProduct
     * @throws \Symfony\Component\Serializer\Exception\ExceptionInterface
     */
    public function getProduct(Product $product, ?Product $productVariation, $country = 'pl'): DtoProduct {

        $attributes = self::getAttributesArray($product, $productVariation);

        if ($productVariation) {
            $price = $this->taxHelper->getTaxPrice($productVariation, $productVariation->getFinalPrice(), true);
        } else {
            $price = $this->taxHelper->getTaxPrice($product, $product->getFinalPrice(), true);
        }

        $cashbenePrice = new Price();
        $cashbenePrice->amount = $price;
        $cashbenePrice->setHasTax(false);
        $cashbenePrice->currency = $this->storeManager->getStore()->getCurrentCurrency()->getCode();

        $cashbeneProduct = new DtoProduct();
        $cashbeneProduct->productId = $product->getId();
        $cashbeneProduct->sku = $productVariation ? $productVariation->getSku() : $product->getSku();
        $cashbeneProduct->externalProductId = $productVariation ? $productVariation->getId() : $product->getId();
        $cashbeneProduct->unitPrice = $cashbenePrice;
        $cashbeneProduct->unitGrossPrice = clone $cashbenePrice;

        $cashbeneProduct->name = $product->getName();
        $cashbeneProduct->image = $this->getProductImage($productVariation ?? $product);

        if ($attributes) {
            $cashbeneProduct->setAllAttributes($attributes);
            $cashbeneProduct->attributes = SerializerService::getSerializer()->denormalize($attributes, Attribute::class.'[]','array');
        }

        return $cashbeneProduct;
    }

    private function getProductImage($product) {
        $imageUrl = $this->imageHelper->init($product, 'product_page_image')
            ->setImageFile($product->getImage()) // image,small_image,thumbnail
            ->getUrl();

        return $imageUrl;
    }

    /**
     * @param $id
     * @return Product|null
     */
    public function getMagentoProduct(int $id)
    {
        $product = null;

        if ($this->productRepository->getById($id)) {
            $product = $this->productRepository->getById($id);
        };

        return $product;
    }

    /**
     * @param $id
     * @return Product|null
     */
    public function getMagentoProductConfigurableVariation(int $id, $variations = null)
    {
        if ($this->productRepository->getById($id)) {
            $product = $this->productRepository->getById($id);
            if ($product->getTypeId() === 'configurable' && $variations) {
                return $this->configurableProduct->getProductByAttributes($variations, $product);
            }
        };

        return null;
    }

    /**
     * @param Product $product
     * @param $productVariation
     * @return array
     */
    public static function getAttributesArray(Product $product, $productVariation) : array
    {
        $attributes = [];
        if ($productVariation && $product !== $productVariation) {
            $data = $product->getTypeInstance()->getConfigurableOptions($product);
            foreach($data as $attrId => $attr){
                foreach($attr as $p){
                    if ($p['sku'] === $productVariation->getSku()) {
                        $attributes[] = [
                            'attr_id' => $attrId,
                            'name' => $p['attribute_code'],
                            'value' => $p['option_title'],
                            'value_id' => $p['value_index'],
                            'partOfName' => false
                        ];
                    }
                }
            }
        }

        return $attributes;
    }

    public function getRegionsByCountryCode($countryCode) : array
    {
        $regionCode = $this->regionCollectionFactory->create()
            ->addCountryFilter($countryCode)
            ->toArray()
        ;
        return $this->prepareRegionArray($regionCode['items']);
    }


    private function prepareRegionArray($regions)
    {
        $array = [];
        foreach ($regions as $region) {
            $array[$region['region_id']]['id'] = $region['region_id'];
            $array[$region['region_id']]['value'] = $region['default_name'];
            $array[$region['region_id']]['country'] = $region['country_id'];
        }

        return $array;
    }
}
